#! /usr/bin/python3 -sP

"""
This module handles the configuration for the JamJar CLI.

This includes reading the configuration file and retrieving the client_id,
client_secret, redirect_uri, database path, and token file path.
"""

import json
import os

CONFIG_FILE = os.path.expanduser("~/.jamjar_config.json")


class Config:
    """
    Handles the configuration for the JamJar CLI.

    This includes reading the configuration file and retrieving the client_id,
    client_secret, redirect_uri, database path, and token file path.
    """

    def __init__(self):
        if not os.path.exists(CONFIG_FILE):
            raise FileNotFoundError(f"Configuration file not found at {CONFIG_FILE}")

        with open(CONFIG_FILE, "r", encoding="utf-8") as f:
            self.config_data = json.load(f)

    def get(self, key, default=None):
        """Get a configuration value by key."""
        if key not in self.config_data:
            if default is None:
                raise KeyError(f"Configuration key '{key}' not found")
        return self.config_data.get(key, default)

    @property
    def client_id(self):
        """Retrieve the client_id from the config."""
        return self.get("client_id")

    @property
    def client_secret(self):
        """Retrieve the client_secret from the config."""
        return self.get("client_secret")

    @property
    def redirect_uri(self):
        """Retrieve the redirect URI from the config."""
        return self.get("redirect_uri", "http://localhost:5000/callback")

    @property
    def db_path(self):
        """Retrieve the database path from the config."""
        return self.get("db_path", "~/.jamjar.db")

    @property
    def token_file(self):
        """Retrieve the token file path from the config."""
        return self.get("token_file", "~/.jamjar_token.json")
